/**************************************************************************
Note: this material was downloaded from the Hayes BBS...
 * ESPFUNC.C                                                              *
 * Defines the ESP interface functions                                    *
 **************************************************************************/

/*** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***
 ***                                                               ***
 ***                  CONFIDENTIAL INFORMATION                     ***
 ***                  ------------------------                     ***
 ***    This Document contains Confidential Information or         ***
 ***    Trade Secrets, or both, which are the property of Hayes    ***
 ***    Microcomputer Products  Inc.  This Document may not be     ***
 ***    copied, reproduced, reduced to any electronic medium or    ***
 ***    machine readable form or otherwise duplicated and the      ***
 ***    information herein may not be used, disseminated or        ***
 ***    otherwise disclosed, except with the prior written consent ***
 ***    of Hayes Microcomputer Products, Inc.                      ***
 ***                                                               ***
 *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***/

/*
    Copyright 1991 Hayes Microcomputer Products, Inc.
    All rights reserved.
    Use of copyright notice does not imply publication.
 */

/********************************* INCLUDES *******************************/

#include  "espfunc.h"
#include  <conio.h>
#include  <stdio.h>
#include  <io.h>
#include  <dos.h>

/**************************** GLOBAL VARIABLES ****************************/
WORD  enhanced_base;
WORD  service_id;
WORD  status_1;
WORD  command_1;
WORD  status_2;
WORD  command_2;
BYTE  port_offset;
WORD  rx_reg;
WORD  tx_reg;
BYTE  rx_rdy;
BYTE  tx_rdy;
BYTE  rx_fifo;
BYTE  tx_fifo;
BYTE  error;
WORD  bit_rate;
BYTE  dmachan;
BYTE  irq;
BYTE  computer;
WORD  register_ready;

/********************************* DEFINES ********************************/

/* DMA REGISTER ADDRESSES AND COMMANDS */
/* PS/2 MACHINES */
#define  FUN_REG           0x18
#define  FUN_EX_REG        0x1A
#define  MEM_ADDR_REG      0x20
#define  MEM_ADDR_REG_READ 0x30
#define  XFER_COUNT_REG    0x40
#define  MODE_REG          0x70
#define  READ_MODE         0x04
#define  WRITE_MODE        0x0C
#define  MASK_REG_SET      0x90
#define  MASK_REG_CLEAR    0xA0
/* IBM AT AND IBM MODEL 30 */
#define  MASK_REG          0x0A
#define  CLEAR_POINTER_REG 0x0C
#define  AT_MODE_REG       0x0B
#define  CH1_MEM_ADDR_REG  0x02
#define  CH3_MEM_ADDR_REG  0x06
#define  CH1_PAGE_REG      0x83
#define  CH3_PAGE_REG      0x82
#define  CH1_COUNT_REG     0x03
#define  CH3_COUNT_REG     0x07
#define  CH1_WRITE         0x05
#define  CH3_WRITE         0x07
#define  CH1_READ          0x09
#define  CH3_READ          0x0B

/* ESP COMMAND CODES */
#define  RESET             0x00
#define  GETTEST           0x01
#define  GETDIPS           0x02
#define  SETIRQ            0x04
#define  SETDMATIME        0x05
#define  SETSRMSK          0x06
#define  SETERRORMSK       0x07
#define  SETFCTYP          0x08
#define  SETFCCHAR         0x09
#define  SETFLOLEV         0x0A
#define  SETTRIG           0x0B
#define  SETRXTIME         0x0C
#define  SETFLOTIME        0x0D
#define  WRITEUART         0x0E
#define  READUART          0x0F
#define  SETPMODE          0x10
#define  CLEARDMASR        0x11
#define  GETERRS           0x12
#define  GETUARTS          0x13
#define  GETRXBYTES        0x14
#define  GETTXBYTES        0x15
#define  DMARX             0x16
#define  DMATX             0x17
#define  XOFFLOCAL         0x18
#define  XONLOCAL          0x19
#define  LINEBREAK         0x1A
#define  FLUSHRX           0x1B
#define  FLUSHTX           0x1C
#define  SETBAUD           0x1D
#define  GETMODE           0x1E
                           
/* OTHER STUFF */
#define  UART_CLOCK        1843200/16


/********************************** MACROS *********************************/

/* ================================ WAITRDY ================================
 * Tests the specified bit in the Register Ready Register until it
 * is set indicating that ESP is ready.
 * BYTE - bit in Register Ready Register to test
 * RETURN:      None
 */
#define  waitrdy( reg_rdy_val )  \
   while ( ( (WORD) inp (register_ready) & reg_rdy_val ) == 0 )


/************************* GENERAL ESP FUNCTIONS ***********************/

/* ============================== INIT ==============================
 * Initialize global ESP & ESP port variables
 * Enter:  none
 * Return: some global variables initialized
 */
void  init( )
{
   register_ready = enhanced_base + 0;
   service_id = enhanced_base + 1;
   status_1 = enhanced_base + 4;
   command_1 = enhanced_base + 4;
   status_2 = enhanced_base + 5;
   command_2 = enhanced_base + 5;
}

/* ============================== SET_PORT ==============================
 * Initialize ESP variables to point to port 1 or port 2
 * Enter:  BYTE = 1 for port 1 or 2 for port 2
 * Return: none
 */
void  set_port( port )
BYTE  port;
{
   if ( port == 1 )
      {
      rx_reg = enhanced_base + 2;      /* rx1 register */
      rx_rdy = RX1_RDY;                /* rx1 ready bit */
      tx_reg = enhanced_base + 2;      /* tx1 register */
      tx_rdy = TX1_RDY;                /* tx1 ready bit */
      rx_fifo = RX1_FIFO_SR;           /* rx1 fifo s.r. reached */
      tx_fifo = TX1_FIFO_SR;           /* tx1 fifo s.r. reached */
      error = P1_ERROR;                /* port 1 error status */
      port_offset = 0;                 /* adder to make port 1 command */
      }
   else     /* port = 2 */
      {
      rx_reg = enhanced_base + 3;      /* rx2 register */
      rx_rdy = RX2_RDY;                /* rx2 ready bit */
      tx_reg = enhanced_base + 3;      /* tx2 register */
      tx_rdy = TX2_RDY;                /* tx2 ready bit */
      rx_fifo = RX2_FIFO_SR;           /* rx2 fifo s.r. reached */
      tx_fifo = TX2_FIFO_SR;           /* tx2 fifo s.r. reached */
      error = P2_ERROR;                /* port 2 error status */
      port_offset = 0x80;              /* adder to make port 2 command */
      }
}

/* ============================= INITIATE_DMA =============================
 * Initiates a host DMA transfer
 * Scott Swanson           6/15/89
 * ENTER:   *char -  transfer buffer
 *          int -    number of characters to be transferred, 1 to 1023
 *          BYTE -   READ = DMA read, WRITE = DMA write
 * RETURN:  none
 */

void  initiate_dma( buffer, byte_count, read_write )
char  *buffer;
int   byte_count;
BYTE  read_write;

{
   BYTE      memory_address_reg;
   BYTE      page_table_address_reg;
   BYTE      transfer_count_reg;
   BYTE      mode_cmd;
   char far  *ptr;
   unsigned  offset;
   unsigned  segment;
   long      physical_address;
   BYTE      dma_read_write_cmd;

   /* DO DIFFERENT INITIATION FOR AT OR PS/2 TYPE BUSSES */
   if (computer == AT )
   {
   /* DO AT DMA INITIATION */

      /* GET THE REGISTER ADDRESSES AND MODE FOR THIS DMA CHANNEL */
      if ( dmachan == 1 )
      {
         memory_address_reg = CH1_MEM_ADDR_REG;
         page_table_address_reg = CH1_PAGE_REG;
         transfer_count_reg = CH1_COUNT_REG;
         if ( read_write == READ )
            mode_cmd = CH1_READ;
         else
            mode_cmd = CH1_WRITE;
      }
      else     /* dmachan == 3 */
      {
         memory_address_reg = CH3_MEM_ADDR_REG;
         page_table_address_reg = CH3_PAGE_REG;
         transfer_count_reg = CH3_COUNT_REG;
         if ( read_write == READ )
            mode_cmd = CH3_READ;
         else
            mode_cmd = CH3_WRITE;
      }

      /* DISABLE TRANSFERS FOR THIS DMA CHANNEL DURING SETUP */
      outp( MASK_REG, 0x04 + dmachan );    /* bit 2 is set */

      /* OUTPUT THE MEMORY ADDRESS TO THE DMA CONTROLLER */
      ptr = buffer;
      offset = FP_OFF( ptr );
      segment = FP_SEG( ptr );
      physical_address = ( (long) segment << 4 ) + (long) offset;   
      outp( CLEAR_POINTER_REG, 0x00 );
      outp( memory_address_reg, (BYTE) ( physical_address & 0x000000FF ) );
      outp( memory_address_reg,
            (BYTE) (( physical_address >> 8 ) & 0x000000FF ) );
      outp( page_table_address_reg,
            (BYTE) (( physical_address >> 16 ) & 0x000000FF ) );

      /* OUTPUT THE TRANSFER COUNT TO THE DMA CONTROLLER */
      --byte_count;    /* the dma controller adds 1 to the count */
      outp( CLEAR_POINTER_REG, 0x00 );
      outp( transfer_count_reg, (BYTE) ( byte_count & 0x00FF ) );
      outp( transfer_count_reg, (BYTE) (( byte_count >> 8 ) & 0x00FF ) );

      /* SPECIFY READ OR WRITE TRANSFER FOR THIS CHANNEL */
      outp( AT_MODE_REG, mode_cmd );

      /* RE-ENABLE TRANSFERS FOR THIS DMA CHANNEL */
      outp( MASK_REG, dmachan );
   }

   else
   { 
   /* DO PS2 DMA INITIATION */

      /* DISABLE TRANSFERS FOR THIS DMA CHANNEL DURING SETUP */
      outp( FUN_REG, MASK_REG_SET + dmachan );

      /* OUTPUT THE MEMORY ADDRESS TO THE DMA CONTROLLER */
      ptr = buffer;
      offset = FP_OFF( ptr );
      segment = FP_SEG( ptr );
      physical_address = ( (long) segment << 4 ) + (long) offset;
      outp( FUN_REG, MEM_ADDR_REG + dmachan );
      outp( FUN_EX_REG, (BYTE) ( physical_address & 0x000000FF ) );
      outp( FUN_EX_REG, (BYTE) (( physical_address >> 8 ) & 0x000000FF ) );
      outp( FUN_EX_REG, (BYTE) (( physical_address >> 16 ) & 0x000000FF ) );

      /* OUTPUT THE TRANSFER COUNT TO THE DMA CONTROLLER */
      --byte_count;     /* the dma controller adds 1 to the count */
      outp( FUN_REG, XFER_COUNT_REG + dmachan );
      outp( FUN_EX_REG, (BYTE) ( byte_count & 0x00FF ) );
      outp( FUN_EX_REG, (BYTE) (( byte_count >> 8 ) & 0x00FF));

      /* SPECIFY READ OR WRITE TRANSFER */
      if ( read_write == READ )
         dma_read_write_cmd = READ_MODE;
      else
         dma_read_write_cmd = WRITE_MODE;
      outp( FUN_REG, MODE_REG + dmachan );
      outp( FUN_EX_REG, dma_read_write_cmd );

      /* RE-ENABLE TRANSFERS FOR THIS DMA CHANNEL */
      outp( FUN_REG, MASK_REG_CLEAR + dmachan );
   }
}

/* ============================= SET_BAUD_RATE =============================
 * Sets the baud rate in the Uart (ESI Command 41H).
 * Enter:
 *    WORD  the actual uart bit rate
 * Return:
 *    None.  No errors reported.
 */
void  set_baud_rate( bit_rate )
WORD  bit_rate;
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, SETBAUD + port_offset );

   waitrdy( COMMAND2_RDY );
   outp( command_2, (BYTE)(UART_CLOCK/bit_rate>>8) ); /* Divisor Latch MSB */
   
   waitrdy( COMMAND2_RDY );
   outp( command_2, (BYTE)(UART_CLOCK/bit_rate) );    /* Divisor Latch LSB */
}


/* ======================= SET_LINE_CHARACTERISTICS =======================
 * Sets the UART Line Control Register stuff (ESI Command 42H)
 * Side Effects:  Clears Break and Divisor Latch Enable bits
 * Enter:
 *    BYTE  serial word size: 5 = 5 bits, 6 = 6 bits, 7 = 7 bits, 8 = 8 bits
 *    BYTE  parity: 0 = none, 1 = odd, 2 = even, 3 = mark, 4 = space
 *    BYTE  stop bits: 0 = 1 stop bit, 1 = 1.5 stop bits, 2 = 2 stop bits
 * Return:
 *    None.  No errors reported.
 */
void  set_line_characteristics( data_bits, parity, stop_bits )
BYTE  data_bits;
BYTE  parity;
BYTE  stop_bits;
{
   BYTE  temp_lcr;

   /* BUILD LINE CONTROL REGISTER */
   temp_lcr = data_bits - 5;
   if (parity != 0)
      temp_lcr |= (parity - 1) * 0x10 + 0x8;
   if (stop_bits != 0)
      temp_lcr |= 0x4;

   /* WRITE LINE CONTROL REGISTER TO UART */
   writeuart( LCR, temp_lcr );
}

/**************** FUNCTIONS TO ISSUE ESP HARDWARE COMMANDS **************/

/* ================================ RESET ================================
 *  PURPOSE:     Resets the ESP
 *  PARAMETERS:  None
 *  RETURN:      None
 */
void  reset( )
{
   inp( status_1 );           /* in case old status data were present */
   waitrdy( COMMAND1_RDY );
   outp( command_1, RESET );
   waitrdy( STATUS1_RDY );
   inp( status_1 );
}

/* ================================ GETTEST ================================
 *  PURPOSE:     Get Self Test Results and ESP Firmware Revision
 *  PARAMETERS:  None
 *  RETURN:      WORD - MSB = firmware revision from status 1 register
 *	                     LSB = self test results from status 2 register
 */
WORD  gettest( )
{
   BYTE  byte;
   WORD  test;

   waitrdy( COMMAND1_RDY );
   outp( command_1, GETTEST );

   /* GET THE REV LEVEL = MSB */
   waitrdy( STATUS1_RDY );
   byte = (BYTE) inp( status_1 );
  	test = (WORD) byte << 8;

   /* GET THE TEST RESULTS = LSB */
  	waitrdy ( STATUS2_RDY );
  	byte = (BYTE) inp( status_2 );
  	test = test + (WORD) byte;

   return( test );
}

/* ================================ GETDIPS ================================
 *  PURPOSE:     Get Unehanced Mode Address DIPs
 *  PARAMETERS:  None
 *  RETURN:      BYTE - Address DIP byte
 */
BYTE  getdips( )
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, GETDIPS );

   waitrdy( STATUS1_RDY );
   return( (BYTE) inp( status_1 ) );

}

/* ================================ SETIRQ =================================
 * Set Enhanced Mode Interrupt Enable and DMA Arbitration Level
 * Enter:  BYTE - bit pattern
 * Return: None
 */
void  setirq( irq_dma )
BYTE  irq_dma;
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, SETIRQ );

   waitrdy( COMMAND2_RDY );
   outp( command_2, irq_dma );
}

/* =============================== SETDMATIME ==============================
 * Sets the DMA timeout value
 * Enter:  BYTE - clock ticks, where 1 clock tick = 10 milliseconds
 * Return: None
 */
void  setdmatime( time )
BYTE  time;
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, SETDMATIME );

   waitrdy( COMMAND2_RDY );
   outp( command_2, time );
}

/* ================================ SETSRMSK ===============================
 * Set Service Request Mask
 * Enter:  BYTE - mask bit pattern
 * Return: None
 */
void  setsrmsk( mask )
BYTE  mask;
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, SETSRMSK );

   waitrdy( COMMAND2_RDY );
   outp( command_2, mask );
}

/* ================================ SETERRORMSK =============================
 * Set Error Status Mask
 * Enter:
 *    WORD - MSB = byte 1 of bit pattern, LSB = byte 2 of bit pattern
 *  RETURN:      None
 */
void  seterrormsk( mask )
WORD  mask;
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, SETERRORMSK + port_offset );

   waitrdy( COMMAND2_RDY );
   outp( command_2, (BYTE)(mask>>8) );

   waitrdy( COMMAND2_RDY );
   outp( command_2, (BYTE) mask );
}

/* ================================ SETFCTYP ===============================
 * Set the flow control type
 * Enter:  BYTE, BYTE - 2 byte bit pattern specifying desired flow control
 * Return: None
 */
void  setfctyp( byte1, byte2 )
BYTE  byte1, byte2;
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, SETFCTYP + port_offset );

   waitrdy( COMMAND2_RDY );
   outp( command_2, byte1 );

   waitrdy( COMMAND2_RDY );
   outp( command_2, byte2 );
}

/* =============================== SETFCCHAR ===============================
 * Set the flow control characters used by XON/XOFF and
 * transparent XON/XOFF flow control.
 * Enter:
 *    BYTE - XON character
 *    BYTE - XOFF character
 *    BYTE - Escape character used by transparent flow
 *    BYTE - XOR character used by transparent flow
 *    BYTE - flow character parity mask
 * Return:  None
 */
void  setfcchar( xon, xoff, dle, xor, pmask )
BYTE  xon;
BYTE  xoff;
BYTE  dle;
BYTE  xor;
BYTE  pmask;
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, SETFCCHAR + port_offset );

   waitrdy( COMMAND2_RDY );
   outp( command_2, xon );

   waitrdy( COMMAND2_RDY );
   outp( command_2, xoff );

   waitrdy( COMMAND2_RDY );
   outp( command_2, dle );

   waitrdy( COMMAND2_RDY );
   outp( command_2, xor );

   waitrdy( COMMAND2_RDY );
   outp( command_2, pmask );
}

/* =============================== SETFLOLEV ===============================
 * Set Receive FIFO Flow Control Levels
 * Enter:
 *    WORD - flow off level
 *    WORD - flow on level
 * Return:  None
 */
void  setflolev( offlev, onlev )
WORD  offlev;
WORD  onlev;
{
   BYTE  byte;

   waitrdy( COMMAND1_RDY );
   outp( command_1, SETFLOLEV + port_offset );

   /* SEND THE MOST SIGNIFICANT BYTE OF THE FLOW OFF LEVEL */
   byte = (BYTE) (( offlev >> 8 ) & 0x00FF );
   waitrdy( COMMAND2_RDY );
   outp( command_2, byte );

   /* SEND THE LEAST SIGNIFICANT BYTE OF THE FLOW OFF LEVEL */
   byte = (BYTE) ( offlev & 0x00FF );
   waitrdy( COMMAND2_RDY );
   outp( command_2, byte );

   /* SEND THE MOST SIGNIFICANT BYTE OF THE FLOW ON LEVEL */
   byte = (BYTE) (( onlev >> 8 ) & 0x00FF );
   waitrdy( COMMAND2_RDY );
   outp( command_2, byte );

   /* SEND THE LEAST SIGNIFICANT BYTE OF THE FLOW ON LEVEL */
   byte = (BYTE) ( onlev & 0x00FF );
   waitrdy( COMMAND2_RDY );
   outp( command_2, byte );
}

/* ================================ SETTRIG ================================
 * Set FIFO Trigger levels which will cause ESP to generate
 * a service request
 * Enter:
 *    WORD - receive trigger level
 *    WORD - transmit trigger level 
 * Return:  None
 */
void  settrig( rxlev, txlev )
WORD  rxlev;
WORD  txlev;
{
   BYTE  byte;

   waitrdy( COMMAND1_RDY );
   outp( command_1, SETTRIG + port_offset );

   /* SEND THE MOST SIGNIFICANT BYTE OF THE RECEIVE TRIGGER LEVEL */
   byte = (BYTE) (( rxlev >> 8 ) & 0x00FF );
   waitrdy( COMMAND2_RDY );
   outp( command_2, byte );

   /* SEND THE LEAST SIGNIFICANT BYTE OF THE RECEIVE TRIGGER LEVEL */
   byte = (BYTE) ( rxlev & 0x00FF );
   waitrdy( COMMAND2_RDY );
   outp( command_2, byte );

   /* SEND THE MOST SIGNIFICANT BYTE OF THE TRANSMIT TRIGGER LEVEL */
   byte = (BYTE) (( txlev >> 8 ) & 0x00FF );
   waitrdy( COMMAND2_RDY );
   outp( command_2, byte );

   /* SEND THE LEAST SIGNIFICANT BYTE OF THE TRANSMIT TRIGGER LEVEL */
   byte = (BYTE) ( txlev & 0x00FF );
   waitrdy( COMMAND2_RDY );
   outp( command_2, byte );
}

/* =============================== SETRXTIME ===============================
 * Set Receive Character Timeout
 * Enter:
 *    BYTE - clock ticks, where 1 clock tick = 4 milliseconds
 * Return:  None
 */
void  setrxtime( time )
BYTE  time;
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, SETRXTIME + port_offset );

   waitrdy( COMMAND2_RDY );
   outp( command_2, time );
}

/* =============================== SETFLOTIME ==============================
 * Set Flowed Off Timeout
 * Enter:
 *    BYTE - time in seconds
 * Return:  None
 */
void  setflotime( time )
BYTE  time;
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, SETFLOTIME + port_offset );

   waitrdy( COMMAND2_RDY );
   outp( command_2, time );
}

/* =============================== WRITEUART ===============================
 * Output a Byte Directly to the UART               
 * Enter:
 *    BYTE - the selected UART register
 *    BYTE - the data to write
 * Return:  None
 */
void  writeuart( ureg, outbyte )
BYTE  ureg;
BYTE  outbyte;

{
   waitrdy( COMMAND1_RDY );
   outp( command_1, WRITEUART + port_offset );

   waitrdy( COMMAND2_RDY );
   outp( command_2, ureg );

   waitrdy( COMMAND2_RDY );
   outp( command_2, outbyte );
}

/* =============================== READUART ================================
 * Input a Byte Directly from the UART                 
 * Enter:  BYTE - the selected UART register
 * Return: BYTE - the data from the UART
 */
BYTE  readuart( ureg )
BYTE  ureg;

{
   BYTE  inbyte;

   waitrdy( COMMAND1_RDY );
   outp( command_1, READUART + port_offset );

   waitrdy( COMMAND2_RDY );
   outp( command_2, ureg );

   waitrdy( STATUS1_RDY );
   inbyte = (BYTE) inp( status_1 );

	return( inbyte );
}

/* ================================ GETMODE ================================
 *  PURPOSE:     Get ESP mode
 *  PARAMETERS:  None
 *  RETURN:      WORD - MSB = default mode from status 1 register
 *	                     LSB = current mode from status 2 register
 */
WORD  getmode( )
{
   BYTE  byte;
   WORD  mode;

   waitrdy( COMMAND1_RDY );
   outp( command_1, GETMODE + port_offset );

   /* GET THE DEFAULT MODE = MSB */
   waitrdy( STATUS1_RDY );
   byte = (BYTE) inp( status_1 );
  	mode = (WORD) byte << 8;

   /* GET THE CURRENT MODE = LSB */
  	waitrdy ( STATUS2_RDY );
  	byte = (BYTE) inp( status_2 );
  	mode = mode + (WORD) byte;

   return( mode );
}

/* =============================== SETPMODE =================================
 * Enter the ESP mode specifed by given bit pattern
 * Enter:
 *    BYTE - bit pattern specifying mode
 * Return:  None
 */
void  setpmode( pmode )
BYTE  pmode;
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, SETPMODE + port_offset );

   waitrdy( COMMAND2_RDY );
   outp( command_2, pmode );
}

/* ================================ CLEARDMASR ============================
 * Clear DMA service request
 * Enter:  None
 * Return: None
 */
void  cleardmasr( )
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, CLEARDMASR );
}

/* ================================ GETERRS ================================
 * Get Error Status
 * Enter:  None
 * Return: WORD - MSB = error status from Status1, 
 *                LSB = error status from Status2
 */
WORD  geterrs( )
{
   BYTE  byte;
   WORD  errs;

   waitrdy( COMMAND1_RDY );
   outp( command_1, GETERRS + port_offset );

   /* GET THE MOST SIGNIFICANT BYTE */
   waitrdy( STATUS1_RDY );
   byte = (BYTE) inp( status_1 );
   errs = (WORD) byte << 8;

   /* GET THE LEAST SIGNIFICANT BYTE */
   waitrdy( STATUS2_RDY );
   byte = (BYTE) inp( status_2 );
   errs = errs + (WORD) byte;

   return( errs );
}

/* ================================ GETUARTS ===============================
 * Get Uart Status
 * Enter:  none
 * Return: WORD - MSB = UART Line Status Register, 
 *                LSB = UART Modem Status Register
 */
WORD  getuarts( )
{
   WORD  uarts;
	BYTE  byte;

   waitrdy( COMMAND1_RDY );
   outp( command_1, GETUARTS + port_offset );

   /* GET THE MOST SIGNIFICANT BYTE */
   waitrdy( STATUS1_RDY );
   byte = (BYTE) inp( status_1 );
   uarts = (WORD) byte << 8;

   /* GET THE LEAST SIGNIFICANT BYTE */
   waitrdy( STATUS2_RDY );
   byte = (BYTE) inp( status_2 );
   uarts = uarts + (WORD) byte;

   return( uarts );
}

/* =============================== GETRXBYTES ==============================
 * Get the number of recevied bytes available in the receive FIFO buffer
 * Enter:  none
 * Return: WORD - the number of received bytes available in the receive
 *                FIFO buffer + the RX register
 */
WORD  getrxbytes( )
{
   BYTE  byte;
   WORD  rxbytes;

   /* COUNT THE BYTE IN THE RX REGISTER, IF AVAILABLE */
   if( inp( register_ready ) & rx_rdy ) 
      rxbytes = 1;
   else rxbytes = 0;

   /* ISSUE THE GETRXBYTES COMMAND */
   waitrdy( COMMAND1_RDY );
   outp( command_1, GETRXBYTES + port_offset );

   /* GET THE MOST SIGNIFICANT BYTE */
   waitrdy( STATUS1_RDY );
   byte = (BYTE) inp( status_1 );
   rxbytes = rxbytes + (WORD) (byte << 8);

   /* GET THE LEAST SIGNIFICANT BYTE */
   waitrdy( STATUS2_RDY );
   byte = (BYTE) inp( status_2 );
   rxbytes = rxbytes + (WORD) byte;

   return( rxbytes );
}

/* =============================== GETTXBYTES ==============================
 * Get the nuber of bytes of space available in the transmit FIFO buffer
 * Enter:  none
 * Return: WORD - the number of bytes of space available in the
 *                transmit FIFO buffer
 */
WORD  gettxbytes( )
{
   BYTE  byte;
   WORD  txbytes;

   waitrdy( COMMAND1_RDY );
   outp( command_1, GETTXBYTES + port_offset );

   /* GET THE MOST SIGNIFICANT BYTE */
   waitrdy( STATUS1_RDY );
   byte = (BYTE) inp( status_1 );
   txbytes = (WORD) byte << 8;

   /* GET THE LEAST SIGNIFICANT BYTE */
   waitrdy( STATUS2_RDY );
   byte = (BYTE) inp( status_2 );
   txbytes = txbytes + (WORD) byte;

   return( txbytes );
}

/* ================================= DMARX =================================
 * Initiate DMA Receive
 * Enter:  none
 * Return: none
 */
void  dmarx( )
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, DMARX + port_offset );
   waitrdy( STATUS1_RDY );
   inp( status_1 );
}

/* ================================= DMATX =================================
 * Initiate DMA Transmit
 * Enter:  none
 * Return: none
 */
void  dmatx( )
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, DMATX + port_offset );
   waitrdy( STATUS1_RDY );
   inp( status_1 );
}

/* ================================= XOFFLOCAL =============================
 * Causes ESP to stop transmitting
 * Enter:  none
 * Return: none
 */
void  xofflocal( )
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, XOFFLOCAL + port_offset );
}

/* ================================= XONLOCAL ==============================
 * Causes ESP to resume transmitting
 * Enter:  none
 * Return: none
 */
void  xonlocal( )
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, XONLOCAL + port_offset );
}

/* =============================== LINEBREAK ===============================
 * Issue a serial line break for the specified break duration
 * Enter:  BYTE - clock ticks, where 1 clock tick = 10 milliseconds
 * Return: none
 */
void  linebreak( ticks )
BYTE  ticks;
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, LINEBREAK + port_offset );

   waitrdy( COMMAND2_RDY );
   outp( command_2, ticks );
}

/* ================================ FLUSHRX ================================
 * Flush the Receive FIFO
 * Enter:  none
 * Return: none
 */
void  flushrx( )
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, FLUSHRX + port_offset );
}

/* ================================ FLUSHTX ================================
 * Flush the Transmit FIFO
 * Enter:  none
 * Return: none
 */
void  flushtx( )
{
   waitrdy( COMMAND1_RDY );
   outp( command_1, FLUSHTX + port_offset );
}

/* ================================ RXBYTE =================================
 * Read Byte from RX Register
 * Enter:  none
 * Return: BYTE - data byte read from RX register
 */
BYTE  rxbyte( )
{
   BYTE  byte;

   waitrdy( rx_rdy );
   byte = (BYTE) inp( rx_reg );

   return( byte );
}

/* ================================ TXBYTE =================================
 * Write Byte to TX Register
 * Enter:  BYTE - the data byte to write
 * Return: none
 */
void  txbyte( byte )
BYTE  byte;
{
   waitrdy( tx_rdy );
   outp( tx_reg, byte );
}

